package com.semaphore;

import org.jdom.*;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;

import java.util.*;

/**
 * Brought to you by:
 * User: semaphore
 * Date: Sep 19, 2009
 * Time: 1:20:48 PM
 */
public class RequestBuilder {
    private static SAXBuilder builder = new SAXBuilder();

    static {
        builder.setValidation(false);
        builder.setIgnoringElementContentWhitespace(true);
        builder.setIgnoringBoundaryWhitespace(true);
        builder.setFeature("http://xml.org/sax/features/validation", false);
        builder.setFeature("http://apache.org/xml/features/validation/schema", false);
        builder.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);

    }

    public static void main(String[] args) throws Exception {
        Document doc = builder.build(RequestBuilder.class.getResourceAsStream("/com/semaphore/data/BuildManifest-iphone-3.0.1.plist"));
        Map<String,String> digests = getPartialDigests(doc);
        System.out.println(digests);
//        XMLOutputter out = new XMLOutputter();
//        out.setFormat(Format.getPrettyFormat());
//        out.output(blobs, System.out);
        
        //XMLOutputter xmlout = new XMLOutputter();
        //xmlout.setFormat(Format.getPrettyFormat());
        //xmlout.output(ndoc, System.out);
    }
    private static boolean validateEcid(String rawEcid) {
        return !(rawEcid == null || rawEcid.length() == 0 || rawEcid.trim().length() == 0) && (rawEcid.matches("[a-fA-F0-9]{11,15}"));
    }


    /**
     * @param version From the BuildVersion Enum
     * @param params  {ecid}
     * @return
     */
    public static Document getRequest(BuildVersion version, String... params) {
        Document request = createDocument();
        Element plist = new Element("plist");
        plist.setAttribute("version", "1.0");
        request.setRootElement(plist);
        Element mdict = new Element("dict");
        plist.addContent(mdict);
        addBlobRequests(mdict,"/com/semaphore/data/BuildManifest-" + version.getVersion() + ".plist", params);
        return request;
    }

    public static Map<String,String[]> getFilePaths(BuildVersion version) {
        try {
            Document doc = builder.build(RequestBuilder.class.getResourceAsStream("/com/semaphore/data/BuildManifest-" + version.getVersion() + ".plist"));
            return getAllPaths(doc);
        } catch(Exception e) {
            e.printStackTrace();
        }
        return new TreeMap<String,String[]>();
    }

    public static Map<String, String> getPartialDigestMap(BuildVersion version) {
        Map<String, String> digests = new TreeMap<String,String>();
        try {
            Document doc = builder.build(RequestBuilder.class.getResourceAsStream("/com/semaphore/data/BuildManifest-" + version.getVersion() + ".plist"));
            digests = getPartialDigests(doc);
        } catch(Exception e) {
            e.printStackTrace();
        }
        return digests;
    }

    private static void addBlobRequests(Element mdict, String resource, String... params) {
        try {
            Document doc = builder.build(RequestBuilder.class.getResourceAsStream(resource));
            Element blobs = getBlobs(doc);
            List keys = blobs.getChildren("key");
            List children = blobs.getChildren("dict");
            XMLOutputter out = new XMLOutputter();
            Format fmt = Format.getPrettyFormat();
            out.setFormat(fmt);
            addIdentities(doc, mdict, params);
            int i = 0;
            for(Object child : children) {
                Element ele = (Element)((Element)keys.get(i++)).clone();
                Element ndict = (Element)((Element)child).clone();
                if(ele.getValue().equalsIgnoreCase("OS"))
                    continue;
                mdict.addContent(ele);
                mdict.addContent(ndict);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    private static Map<String,String> getPartialDigests(Document doc) throws JDOMException {
        XPath digests = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/dict[preceding-sibling::key[.='Manifest']]/dict/data[preceding-sibling::key[.='PartialDigest']]");
        XPath keypath = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/dict[preceding-sibling::key[.='Manifest']]/dict[data[.!='']]/preceding-sibling::key[1]");
        List digestNodes = digests.selectNodes(doc);
        List keyNodes = keypath.selectNodes(doc);
        Map<String,String> paths = new TreeMap<String,String>();
        for(int i = 0; i < keyNodes.size(); i++) {
            String key = ((Element)keyNodes.get(i)).getValue();
            String digest = ((Element)digestNodes.get(i)).getValue().trim().replaceAll("\\s","");
            paths.put(key, digest);
        }
        return paths;
    }

    private static Map<String,String[]> getAllPaths(Document doc) throws JDOMException {
        XPath apath = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/dict[preceding-sibling::key[.='Manifest']]/dict[data[.!='']]/dict/key[.='Path']/following-sibling::string");
        XPath keypath = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/dict[preceding-sibling::key[.='Manifest']]/dict[data[.!='']]/preceding-sibling::key[1]");
        List keyNodes = keypath.selectNodes(doc);
        List nodes = apath.selectNodes(doc);
        XPath digestPath = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/dict[preceding-sibling::key[.='Manifest']]/dict[data[.!='']]/key[.='PartialDigest']/following-sibling::data");
        List digestNodes = digestPath.selectNodes(doc);

        Map<String,String[]> paths = new TreeMap<String,String[]>();
        for(int i = 0; i < keyNodes.size(); i++) {
            String key = ((Element)keyNodes.get(i)).getValue();
            String value = ((Element)nodes.get(i)).getValue();
            String digest = ((Element)digestNodes.get(i)).getValue().trim().replaceAll("[\r]?\n","");
            paths.put(key, new String[] {value, digest});
        }
        return paths;
    }

    private static String getChipId(Document doc) throws JDOMException {
        XPath apath = XPath.newInstance("/plist/dict/array/dict/key[.='ApChipID']/following-sibling::string");
        String hexChipID = ((Element)apath.selectSingleNode(doc)).getValue();
        hexChipID = Integer.toString(Integer.parseInt(hexChipID.substring(2), 16), 10);
        return hexChipID;
    }

    private static String getUniqueBuildID(Document doc) throws JDOMException {
        XPath apath = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/data[preceding-sibling::key[.='UniqueBuildID']]");
        Element ele = (Element)apath.selectSingleNode(doc);
        return ele == null ? "" : ele.getValue().trim();
    }

    private static String getBoardId(Document doc) throws JDOMException {
        XPath apath = XPath.newInstance("/plist/dict/array/dict/key[.='ApBoardID']/following-sibling::string");
        String hexBoardID = ((Element)apath.selectSingleNode(doc)).getValue();
        hexBoardID = Integer.toString(Integer.parseInt(hexBoardID.substring(2), 16), 10);
        return hexBoardID;
    }

    private static Element getBlobs(Document doc) throws JDOMException {
        XPath apath = XPath.newInstance("/plist/dict/array/dict[dict/key[.='RestoreBehavior']/following-sibling::string[.='Erase']]/dict[preceding-sibling::key[.='Manifest']]");
        return (Element)apath.selectSingleNode(doc);
    }

    private static Document createDocument() {
        Document request = new Document();
        request.setDocType(new DocType("plist", "-//Apple Computer//DTD PLIST 1.0//EN", "http://www.apple.com/DTDs/PropertyList-1.0.dtd"));
        return request;
    }

    private static void addIdentities(Document doc, Element mdict, String... values) throws JDOMException {
        Element key = new Element("key");
        key.addContent(new Text("@HostIpAddress"));
        mdict.addContent(key);
        key = new Element("string");
        key.addContent(new Text("192.168.0.1"));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("@HostPlatformInfo"));
        mdict.addContent(key);
        key = new Element("string");
        key.addContent(new Text(getPlatform()));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("@Locality"));
        mdict.addContent(key);
        key = new Element("string");
        key.addContent(new Text(Locale.getDefault() == null ? "en_US" : Locale.getDefault().toString()));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("@VersionInfo"));
        mdict.addContent(key);
        key = new Element("string");
        key.addContent(new Text("3.8"));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("ApBoardID"));
        mdict.addContent(key);
        key = new Element("integer");
        key.addContent(new Text(getBoardId(doc)));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("ApChipID"));
        mdict.addContent(key);
        key = new Element("integer");
        key.addContent(new Text(getChipId(doc)));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("ApECID"));
        mdict.addContent(key);
        key = new Element("string");
        key.addContent(new Text(values[0]));
        mdict.addContent(key);

        key = new Element("key");
        key.addContent(new Text("ApProductionMode"));
        mdict.addContent(key);
        mdict.addContent(new Element("true"));

        key = new Element("key");
        key.addContent(new Text("ApSecurityDomain"));
        mdict.addContent(key);
        key = new Element("integer");
        key.addContent(new Text("1"));
        mdict.addContent(key);

        String ubid = getUniqueBuildID(doc);
        if(ubid.length() > 0) {
            key = new Element("key");
            key.addContent(new Text("UniqueBuildID"));
            mdict.addContent(key);
            key = new Element("data");
            key.addContent(new Text(ubid));
            mdict.addContent(key);
        }
    }

    public static String getPlatform() {
        if(System.getProperty("os.name").toLowerCase().indexOf("indows") > -1)
            return "windows";
        return "darwin";
    }

}
