package com.semaphore;

import com.sun.org.apache.xalan.internal.xsltc.cmdline.getopt.GetOpt;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.jdom.Document;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import javax.swing.*;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.StringWriter;

/**
 * Brought to you by:
 * User: semaphore
 * Date: Sep 15, 2009
 * Time: 9:27:38 PM
 */
public class Umbrella {

    public static void main(String[] args) throws Exception {
        if(args.length == 0) {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            } catch(Exception e) {
            }
            new UmbrellaGUI();
        } else {
            GetOpt g = new GetOpt(args, "d:e:r:f:?");
            int c;
            BuildVersion bver = BuildVersion.IPHONE_3_1;
            SHSHRepo repo = SHSHRepo.APPLE;
            String fileName = "";
            String ecid ="";
            while((c = g.getNextOption()) != -1) {
                switch(c) {
                    case 'd'://optional
                        String version = g.getOptionArg();
                        bver = BuildVersion.valueOf(version.toUpperCase());
                        break;
                    case 'e'://required
                        ecid = g.getOptionArg();
                        break;
                    case 'r'://optional
                        String srepo = g.getOptionArg();
                        repo = SHSHRepo.valueOf(srepo);
                        break;
                    case 'f'://optional
                        fileName = g.getOptionArg();
                        break;
                    case '?':
                    default:
                        printUsage();
                        System.exit(-1);
                }
            }
            Umbrella umbrella = new Umbrella(repo.getServer());
            if(!umbrella.validateEcid(ecid)) {
                System.err.println("Bad ECID:" + ecid);
                System.exit(-2);
            }
            if(fileName.length() == 0)
                fileName = ecid + "-" + bver.getVersion() + ".shsh";
            ecid = umbrella.formatEcid(ecid);
            umbrella.executeRequest(RequestBuilder.getRequest(bver, ecid), fileName);
        }
    }

    private static void printUsage() {
        StringBuffer sb = new StringBuffer();
        for(BuildVersion bv : BuildVersion.values())
            sb.append(bv.name()).append(' ');
        String verString = sb.toString().trim().replace(' ', '|').toUpperCase();
        sb = new StringBuffer();
        for(SHSHRepo shshRepo : SHSHRepo.values())
            sb.append(shshRepo.name()).append(' ');
        String repoString = sb.toString().trim().replace(' ', '|').toLowerCase();
        System.err.println("" +
                "--Umbrella-- v" + UmbrellaData.version + "\n" +
                "Usage:\n" +
                "\tNOTE: executing umbrella with no arguments will start umbrella in GUI mode.\n" +
                "\n" +
                "WINDOWS:\n" +
                "\t(executable) -e [ECID] [-d [device+version] -r [repository] -f [filename]]\n" +
                "MAC:\n" +
                "\tjava -jar umbrella.jar -e [ECID] [-d [device+version] -r [repository] -f [filename]]\n" +
                "\nWHERE:\n" +
                "\tECID = REQUIRED = Your device's ECID\n" +
                "\tdevice-version = OPTIONAL (Default is iphone_3_1) = (" + verString + ") The type + firmware version of your device\n" +
                "\trepository = OPTIONAL (Default is apple) = (" + repoString + ") The location to request your SHSH from (Make sure your hosts file does not have an entry for gs.apple.com!)\n" +
                "\tfilename = OPTIONAL (Default is {ecid}-{device+version}.shsh) = The file name to save your SHSH file\n" +
                "\nEXAMPLES:\n" +
                "\nWINDOWS - Have Cydia grab your 3.1 shsh from apple and store them on Cydia!\n" +
                "\tumbrella -e 123123123123 -d iphone_3_1 -r cydia\n" +
                "\n\tSAME as above but save the SHSH under a different file name my31.shsh\n" +
                "\tumbrella -e 123123123123 -d iphone_3_1 -r cydia -f my31.shsh\n" +
                "\n\tStart umbrella's built-in gui\n" +
                "\tumbrella\n" +
                "\nMAC - Have Cydia grab your 3.1 shsh from apple and store them on Cydia!\n" +
                "\tjava -jar umbrella.jar -e 123123123123 -d iphone_3_1 -r cydia\n" +
                "\n\tSee if Cydia has my 3.0 shsh for my iphone\n" +
                "\tjava -jar umbrella.jar -e 123123123123 -d iphone_3_0 -r cydia\n" +
                "\n\tStart umbrella's built-in gui\n" +
                "\tjava -jar umbrella.jar\n\n");
    }

    private static void writeFile(String s, String data) {
        try {
            File f = new File(s);
            UmbrellaData.backupFile(f);
            FileOutputStream fos = new FileOutputStream(f);
            fos.write(data.getBytes());
            fos.flush();
            fos.close();
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    private HttpClient client;
    private String server;

    public Umbrella() {
        this("gs.apple.com");
    }

    public Umbrella(String server) {
        client = new HttpClient();
        this.server = server;
    }

    private String formatEcid(String rawEcid) {
        long l = -1;
        try {
            l = Long.parseLong(rawEcid);
        } catch(NumberFormatException nfe) {
            try {
                l = Long.parseLong(rawEcid, 16);
            } catch(NumberFormatException nfe2) {
            }
        }
        return Long.toString(l, 10);
    }

    private boolean validateEcid(String rawEcid) {
        if(rawEcid == null || rawEcid.length() == 0 || rawEcid.trim().length() == 0) {
            return false;
        }
        if(rawEcid.matches("[a-fA-F0-9]{11,16}")) {
            try {
                Long.parseLong(rawEcid);
                return true;
            } catch(NumberFormatException nfe) {
                try {
                    Long.parseLong(rawEcid, 16);
                    return true;
                } catch(NumberFormatException nfe2) {
                }
            }
        }
        return false;
    }

    public void executeRequest(Document doc, String fileName) {
        PostMethod pm = new PostMethod("http://" + server + "/TSS/controller?action=2");
        addPostHeaders(pm);
        XMLOutputter xmlOut = new XMLOutputter();
        xmlOut.setFormat(Format.getPrettyFormat());
        StringWriter sw = new StringWriter();
        try {
            xmlOut.output(doc, sw);
        } catch(Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
        String data = sw.toString();
        pm.setRequestEntity(new ByteArrayRequestEntity(data.getBytes()));
        pm.addRequestHeader("Content-Length", "" + data.length());
        String retVal = executeMethod(pm);
        if(retVal.toLowerCase().indexOf("status=0&") == -1) {
            throw new RuntimeException("Could not get your ECID SHSH for that model/version: sorry.");
        } else {
            writeFile(fileName, retVal);
        }
    }

    private String executeMethod(HttpMethodBase mth) {
        String val = "";
        try {
            int returnVal = client.executeMethod(mth);
            if(returnVal != HttpStatus.SC_OK) {
                System.err.println("HTTP ERROR:" + returnVal);
            } else {
                InputStream in = mth.getResponseBodyAsStream();
                byte[] bread = new byte[8192];
                StringBuffer sb = new StringBuffer();
                int read;
                do {
                    read = in.read(bread);
                    if(read > -1)
                        sb.append(new String(bread, 0, read));
                } while(read != -1);
                val = sb.toString();
            }
        } catch(Exception e) {
            e.printStackTrace();
            System.err.println("ERROR: Something ain't right check the values!");
        }
        return val;
    }


    private void addPostHeaders(PostMethod pm) {
        pm.addRequestHeader("Accept", "*/*");
        pm.addRequestHeader("Cache-Control", "no-cache");
        pm.addRequestHeader("Content-type", "text/xml; charset=\"utf-8\"");
        pm.addRequestHeader("User-Agent", "InetURL/1.0");
        pm.addRequestHeader("Host", "gs.apple.com");
    }


}
