package com.semaphore;

import org.jdom.Document;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

/**
 * Brought to you by:
 * User: semaphore
 * Date: Oct 2, 2009
 * Time: 7:49:21 AM
 */
public class UmbrellaGUI extends JFrame {
    private JPanel mainPanel;
    private JTextField ecidField;
    private JComboBox deviceCombo;
    private JButton submitButton;
    private JTextField statusTextField;
    private JComboBox repositoryCombo;

    public UmbrellaGUI() {
        super("The Firmware Umbrella - semaphore v" + UmbrellaData.version);
        add(mainPanel);
        setSize(550, 225);
        setResizable(false);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationRelativeTo(null);
        init();
        setVisible(true);
        ecidField.requestFocus();
    }

    private void init() {
        JMenuBar mb = new JMenuBar();
        JMenu menu = new JMenu("Umbrella");
        JMenuItem help = new JMenuItem("Help");
        JMenuItem about = new JMenuItem("About");

        about.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                handleAbout();
            }
        });

        help.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                handleHelp();
            }
        });
        help.setToolTipText("Click this if you really have no clue wtf you are doing.");

        menu.add(help);
        menu.add(about);
        mb.add(menu);
        setJMenuBar(mb);
        for(SHSHRepo repo : SHSHRepo.values())
            repositoryCombo.addItem(repo);
        repositoryCombo.setSelectedIndex(0);
        repositoryCombo.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                SHSHRepo repo = (SHSHRepo)e.getItem();
                repositoryCombo.setToolTipText(repo.getTipText());
            }
        });
        for(BuildVersion val : BuildVersion.values())
            deviceCombo.addItem(val);
        deviceCombo.setSelectedIndex(0);
        ecidField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                deviceCombo.requestFocus();
            }
        });
        submitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                handleSubmit();
            }
        });
    }

    private void handleAbout() {
        JOptionPane.showMessageDialog(this, "<html><p><b>Umbrella</b> <i>v" + UmbrellaData.version + "</i><br/>" +
                "<p>Created by: semaphore<br/>" +
                "<p>" +
                "<p>Thanks: iGuru, Switch, theheadfl, dtube and others unmentioned...<br/>" +
                "<p>" +
                "<p>License: This code is released under GPL<br/>" +
                "http://www.gnu.org/licenses/gpl.html<br/>" +
                "</html>");
    }

    private void handleHelp() {
        JOptionPane.showMessageDialog(this, "<html><p/>OK<br/>This is very simple:<p/><ul><li>1. Enter your ECID</li><li>2. Select the device/version combo you have (or want to get the shsh blobs for).</li><li>3. Select the repository to grab your shsh from (Apple will not sign 3.0.x anymore and if saurik didn't save yours back in early september you are simply SOL. Sorry)</li><li>4. Click 'Submit'</li></ul></html>", "HELP!", JOptionPane.QUESTION_MESSAGE);
    }

    private void handleSubmit() {
        if(!validateEcid()) {
            statusTextField.setText("That ECID is not valid.");
            return;
        }
        BuildVersion version = (BuildVersion)deviceCombo.getSelectedItem();
        SHSHRepo server = (SHSHRepo)repositoryCombo.getSelectedItem();
        String ecid = formatEcid();
        if(ecid.equals("-1"))
            ecid = "";
        statusTextField.setText("Requesting " + version + " ECID SHSH for [" + ecid + "] from [" + server + "]");

        Document doc = RequestBuilder.getRequest(version, ecid);
        Umbrella umbrella = new Umbrella(server.getServer());
        try {
            umbrella.executeRequest(doc, ecid + "-" + version.getVersion() + ".shsh");
            statusTextField.setText("SHSH Saved! File: " + ecid + "-" + version.getVersion() + ".shsh");
        } catch(RuntimeException rte) {
            statusTextField.setText(rte.getMessage());
        }
    }

    private String formatEcid() {
        String rawEcid = ecidField.getText();
        long l = -1;
        try {
            l = Long.parseLong(rawEcid);
        } catch(NumberFormatException nfe) {
            try {
                l = Long.parseLong(rawEcid, 16);
            } catch(NumberFormatException nfe2) {
            }
        }
        return Long.toString(l, 10);
    }

    private boolean validateEcid() {
        String rawEcid = ecidField.getText();
        if(rawEcid == null || rawEcid.length() == 0 || rawEcid.trim().length() == 0) {
            statusTextField.setText("Your ECID is empty!");
            return false;
        }
        if(rawEcid.matches("[a-fA-F0-9]{6,16}")) {
            try {
                Long.parseLong(rawEcid);
                return true;
            } catch(NumberFormatException nfe) {
                try {
                    Long.parseLong(rawEcid, 16);
                    return true;
                } catch(NumberFormatException nfe2) {
                    statusTextField.setText("Unable to parse that ECID: Check it very carefully!");
                }
            }
        }
        return false;
    }

}
